package com.letterpaw.soap.database.model;

import com.letterpaw.soap.database.Model;

import java.sql.*;
import java.util.ArrayList;
import java.util.Date;

public class Logging extends Model {
  // Table name
  protected static final String TABLE_NAME = "logging";
  // SQL Schema for the table
  protected static final String CREATE_SQL = String.join(System.getProperty("line.separator"),
          "CREATE TABLE IF NOT EXISTS " + TABLE_NAME + " (",
          "    id INT NOT NULL AUTO_INCREMENT,",
          "    description VARCHAR(255) NOT NULL,",
          "    IP VARCHAR(255) NOT NULL,",
          "    endpoint VARCHAR(255) NOT NULL,",
          "    timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,",
          "    PRIMARY KEY (id)",
          ");"
  );

  public int id = -1;
  public String description;
  public String IP;
  public String endpoint;
  public Timestamp timestamp = null;

  public static void init() {
    // Static method to Create Table if table is not exist
    try {
      db.runExecute(CREATE_SQL);
      System.out.println("init logging");
    } catch (SQLException e) {
      System.out.println("error init logging");
      e.printStackTrace();
    }
  }

  public static Logging get(int id) throws SQLException {
    // Static method to Get data
    Logging l = null;
    ArrayList<Logging> res = find(id, -1);
    if (!res.isEmpty())
      l = res.get(0);
    return l;
  }

  public static ArrayList<Logging> getAll() throws SQLException {
    // Static method to Get all data
    return find(-1, -1);
  }

  public static ArrayList<Logging> getAll(int limit) throws SQLException {
    // Static method to Get all data with limit
    return find(-1, limit);
  }

  public static ArrayList<Logging> find(int id, int limit) throws SQLException {
    // Static method to find a data with an id with a limit
    PreparedStatement stmt = db.prep(
            "SELECT * FROM " + TABLE_NAME +
                    (id == -1 ? "" : " WHERE id = ?") +
                    (limit == -1 ? "" : " LIMIT ?")
    );

    if (id == -1) {
      // if id is not set, only set limit
      stmt.setInt(1, limit);
    } else {
      // if id is set, set the id
      stmt.setInt(1, id);
      // if limit is set, then set the limit in the query
      if (limit != -1)
        stmt.setInt(2, limit);
    }
    // execute the query
    ResultSet rs = stmt.executeQuery();
    ArrayList<Logging> res = new ArrayList<>();
    while (rs.next()) {
      // get the response from the result set then add into the list
      Logging logging = new Logging();
      logging.id = rs.getInt("id");
      logging.description = rs.getString("description");
      logging.IP = rs.getString("ip");
      logging.endpoint = rs.getString("endpoint");
      logging.timestamp = rs.getTimestamp("requested_at");
      logging.is_persisted = true;
      res.add(logging);
    }
    stmt.getConnection().close();
    return res;
  }

  public void save() throws SQLException {
    // if not saved, make new timestamp
    if (!this.is_persisted)
      this.timestamp = new Timestamp(new Date().getTime());
    PreparedStatement stmt;
    if (this.is_persisted) {
      // if it is already saved, update the data
      stmt = db.prep(
              "UPDATE " + TABLE_NAME + " SET description=?, IP=?, endpoint=?, timestamp=? WHERE id=?"
      );
      stmt.setInt(5, this.id);
    } else {
      // if not saved, insert the data
      stmt = db.prep(
              "INSERT INTO " + TABLE_NAME + " (description, IP, endpoint, timestamp) VALUES(?,?,?,?)",
              true
      );
    }
    // set the parameter with the data
    stmt.setString(1, this.description);
    stmt.setString(2, this.IP);
    stmt.setString(3, this.endpoint);
    stmt.setTimestamp(4, this.timestamp);
    if (stmt.executeUpdate() == 0) {
      // if not saved
      throw new SQLException("Failed to save log");
    }
    if (!this.is_persisted) {
      // if the action is inserting
      ResultSet rs = stmt.getGeneratedKeys();
      if (rs.next()) {
        // get the generated key for the id
        this.id = rs.getInt(1);
      } else {
        throw new SQLException("Failed to get ID of inserted log.");
      }
      this.is_persisted = true;
    }
    stmt.getConnection().close();
  }

  public void delete() throws SQLException {
    // if not saved, return
    if (!this.is_persisted) return;
    try (PreparedStatement stmt = db.prep("DELETE FROM " + TABLE_NAME + " WHERE id=?")) {
      stmt.setInt(1, this.id);
      if (stmt.executeUpdate() == 0)
        throw new SQLException("Failed to delete logging with id " + this.id);
      this.is_persisted = false;
    }
  }
}
