package com.letterpaw.soap.database.model;

import com.letterpaw.soap.database.Model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

public class UserPremium extends Model {
    // Table name
    protected static final String TABLE_NAME = "userPremium";
    // SQL Schema for the table
    protected static final String CREATE_SQL = String.join(System.getProperty("line.separator"),
            "CREATE TABLE IF NOT EXISTS " + TABLE_NAME + " (",
            "    id INT NOT NULL,",
            "    userEmail VARCHAR(255) NOT NULL,",
            "    premiumStatus VARCHAR(255) NOT NULL,",
            "    PRIMARY KEY (id)",
            ");"
    );

    public enum Status {
        PENDING,
        ACCEPTED,
        REJECTED
    };

    public int id = -1;
    public String userEmail;
    public Status premiumStatus = Status.PENDING;

    public static void init() {
        // Static method to Create Table if table is not exist
        try {
            db.runExecute(CREATE_SQL);
            System.out.println("init userpremium");
        } catch (SQLException e) {
            System.out.println("error init userpremium");
            e.printStackTrace();
        }
    }

    public static UserPremium get(int id) throws SQLException {
        // Static method to Get data
        UserPremium l = null;
        ArrayList<UserPremium> res = find(id, -1);
        if (!res.isEmpty())
            l = res.get(0);
        return l;
    }

    public static ArrayList<UserPremium> getAll() throws SQLException {
        // Static method to Get all data
        return find(-1, -1);
    }

    public static ArrayList<UserPremium> getAll(int limit) throws SQLException {
        // Static method to Get all data with limit
        return find(-1, limit);
    }

    public static ArrayList<UserPremium> find(int id, int limit) throws SQLException {
        // Static method to find a data with an id with a limit
        PreparedStatement stmt = db.prep(
                "SELECT * FROM " + TABLE_NAME +
                        (id == -1 ? "" : " WHERE id = ?") +
                        (limit == -1 ? "" : " LIMIT ?")
        );
        if (id == -1) {
            // if id is not set, only set limit
            if (limit != -1) {
                stmt.setInt(1, limit);
            }
        } else {
            // if id is set, set the id
            stmt.setInt(1, id);
            // if limit is set, then set the limit in the query
            if (limit != -1)
                stmt.setInt(2, limit);
        }
        // execute the query
        ResultSet rs = stmt.executeQuery();
        ArrayList<UserPremium> res = new ArrayList<>();
        while (rs.next()) {
            // get the response from the result set then add into the list
            UserPremium userPremium = new UserPremium();
            userPremium.id = rs.getInt("id");
            userPremium.userEmail = rs.getString("userEmail");
            userPremium.premiumStatus = Status.valueOf(rs.getString("premiumStatus"));
            userPremium.is_persisted = true;
            res.add(userPremium);
        }
        stmt.getConnection().close();
        return res;
    }

    @Override
    public void save() throws SQLException {
        PreparedStatement stmt;
        if (this.is_persisted) {
            // if it is already saved, update the data
            stmt = db.prep(
                    "UPDATE " + TABLE_NAME + " SET userEmail=?, premiumStatus=? WHERE id=?"
            );
            stmt.setInt(3, this.id);
        } else {
            // if not saved, insert the data
            stmt = db.prep(
                    "INSERT INTO " + TABLE_NAME + " (`userEmail`, `premiumStatus`, `id`) VALUES(?,?,?)"
            );
        }
        // set the parameter with the data
        stmt.setString(1, this.userEmail);
        stmt.setString(2, String.valueOf(this.premiumStatus));
        stmt.setInt(3, this.id);
        System.out.println(stmt.toString());
        // if not saved
        if (stmt.executeUpdate() == 0)
            throw new SQLException("Failed to persist user premium");
        if (!this.is_persisted) {
            // if the action is inserting

            this.is_persisted = true;
        }
        stmt.getConnection().close();
    }

    @Override
    public void delete() throws SQLException {
        // if not saved, return
        if (!this.is_persisted) return;
        try (PreparedStatement stmt = db.prep("DELETE FROM " + TABLE_NAME + " WHERE id=?")) {
            stmt.setInt(1, this.id);
            if (stmt.executeUpdate() == 0)
                throw new SQLException(
                        "Failed to delete user premium with id " +
                                this.id
                );
            this.is_persisted = false;
        }
    }

}
