package com.letterpaw.soap.service;

import com.letterpaw.soap.database.model.UserPremium;
import com.letterpaw.soap.enums.ResponseType;
import com.letterpaw.soap.enums.ServiceType;
import com.letterpaw.soap.util.Auth;
import com.letterpaw.soap.util.Logger;
import com.letterpaw.soap.util.Mailer;
import com.letterpaw.soap.util.Request;

import javax.annotation.Resource;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
@WebService(serviceName = "UserPremiumService", endpointInterface = "com.letterpaw.soap.service.UserPremiumService")
public class UserPremiumServiceImpl implements UserPremiumService {

    @Resource
    WebServiceContext context;

    @Override
    @WebMethod
    public String registerPremium(int userId, String userEmail) throws SQLException {
        Logger.log(context,
                "[PHP App] Add PENDING premium request for user ID " + userId, "/userpremiumservice{registerPremium}");
        if (Auth.IsAuthenticated(context, ServiceType.PHP)) {
            UserPremium s = UserPremium.get(userId);
            if (s == null) {
                s = new UserPremium();
                s.id = userId;
                s.userEmail = userEmail;
                s.premiumStatus = UserPremium.Status.PENDING;
                s.save();
                // Notify admin
                Request.GET(ServiceType.PHP, "/admins", (String adminJson) -> {
                    Mailer.notifyAdminNewSub(
                            "There is a new premium request from user id " + userId,
                            adminJson
                    );
                });
//                String adminJson = "[{\"email\":\"ken35kiel@gmail.com\"}]";
//                Mailer.notifyAdminNewSub("There is a new premium request from user ID " + userId, adminJson);
                return ResponseType.SUCCESS.toString(); // Success adding new Successfully registered for premium
            }
            return ResponseType.FAILED.toString(); // User Premium already exist User Premium already exists
        }
        throw new SQLException("Not authenticated");
    }

    @Override
    @WebMethod
    public String checkStatus(int userId) throws SQLException {
        // Logging
        // Auth
        if (Auth.IsAuthenticated(context, ServiceType.REST)) {
            Logger.log(context, "[REST] Check status user for ID " + userId, "/userpremiumservice{checkStatus}");
            UserPremium userPremium = UserPremium.get(userId);
            if (userPremium == null) {
                return ResponseType.UNREGISTERED.toString();
            } else {
                if (userPremium.premiumStatus == UserPremium.Status.PENDING) {
                    return userPremium.premiumStatus.toString();
                } else if (userPremium.premiumStatus == UserPremium.Status.REJECTED){
                    return userPremium.premiumStatus.toString();
                } else {
                    return userPremium.premiumStatus.toString();
                }
            }
        }
        if (Auth.IsAuthenticated(context, ServiceType.PHP)) {
            Logger.log(context, "[PHP App] Check status user for ID " + userId, "/userpremiumservice{checkStatus}");
            UserPremium userPremium = UserPremium.get(userId);
            if (userPremium == null) {
                return ResponseType.UNREGISTERED.toString();
            } else {
                if (userPremium.premiumStatus == UserPremium.Status.PENDING) {
                    return userPremium.premiumStatus.toString();
                } else if (userPremium.premiumStatus == UserPremium.Status.REJECTED){
                    return userPremium.premiumStatus.toString();
                } else {
                    return userPremium.premiumStatus.toString();
                }
            }
        }
        throw new SQLException("Not authenticated");
    }

    @Override
    @WebMethod
    public String cancelRegister(int userId) throws SQLException {
        Logger.log(context, "[PHP App] Cancel Premium Request for ID " + userId, "/userpremiumservice{cancelRegister}");
        if (Auth.IsAuthenticated(context, ServiceType.PHP)) {
            UserPremium s = UserPremium.get(userId);
            if (s == null) {
                return ResponseType.FAILED.toString(); // User is not registered, cannot cancel
            } else {
                s.delete();
                return ResponseType.SUCCESS.toString(); // User Premium Request successfully canceled
            }
        }
       throw new SQLException("Not authenticated");
    }

    @Override
    @WebMethod
    public String approvePremium(int userId) throws SQLException {
        Logger.log(context, "[REST] Approve Premium Request for ID " + userId, "/userpremiumservice{approvePremium}");
        if (Auth.IsAuthenticated(context, ServiceType.REST)) {
            UserPremium s = UserPremium.get(userId);
            if (s == null) {
                throw new SQLException("Request is not present");
            } else {
                s.premiumStatus = UserPremium.Status.ACCEPTED;
                s.save();
                return ResponseType.SUCCESS.toString(); // User Premium Request has been accepted
            }
        }
        throw new SQLException("Not authenticated");
    }

    @Override
    public String rejectPremium(int userId) throws SQLException {
        Logger.log(context, "[REST] Reject Premium Request for ID " + userId, "/userpremiumservice{rejectPremium}");
        if (Auth.IsAuthenticated(context, ServiceType.REST)) {
            UserPremium s = UserPremium.get(userId);
            if (s == null) {
                throw new SQLException("Request is not present");
            } else {
                s.premiumStatus = UserPremium.Status.REJECTED;
                s.save();
                return ResponseType.SUCCESS.toString(); // User Premium Request has been rejected
            }
        }
        throw new SQLException("Not authenticated");
    }

    @Override
    @WebMethod
    public List<UserPremium> getAllPremium() throws SQLException {
        Logger.log(context, "[REST] Get all premium user for admin", "/userpremiumservice{getAllPremium}");
        if (Auth.IsAuthenticated(context, ServiceType.REST)) {
            List<UserPremium> s = UserPremium.find(-1, -1);
            List<UserPremium> userPremiums = new ArrayList<>();
            for (UserPremium user : s) {
                if (user.premiumStatus == UserPremium.Status.ACCEPTED) {
                    userPremiums.add(user);
                }
            }
            return userPremiums;
        }
        throw new SQLException("Not authenticated");
    }

    @Override
    public List<UserPremium> getAllPending() throws SQLException {
        Logger.log(context, "[REST] Get all pending premium request for admin", "/userpremiumservice{getAllPending}");
        if (Auth.IsAuthenticated(context, ServiceType.REST)) {
            List<UserPremium> s = UserPremium.find(-1, -1);
            List<UserPremium> userPremiums = new ArrayList<>();
            for (UserPremium user : s) {
                if (user.premiumStatus == UserPremium.Status.PENDING) {
                    userPremiums.add(user);
                }
            }
            return userPremiums;
        }
        throw new SQLException("Not authenticated");
    }

}
