package com.letterpaw.soap.util;

import io.github.cdimascio.dotenv.Dotenv;
import jakarta.mail.*;
import jakarta.mail.internet.InternetAddress;
import jakarta.mail.internet.MimeMessage;

import java.util.List;
import java.util.Properties;

import org.json.JSONArray;
import org.json.JSONObject;

public class Mailer {
    static Dotenv dotenv = Dotenv.configure().ignoreIfMissing().load();
    static final String from = dotenv.get("MAILER_FROM", "Letterpaw SOAP <noreply@soap.letterpaw.com>");
    // Credentials to log in to the SMTP
    static final String username = dotenv.get("MAILER_USER", "mail-user");
    static final String password = dotenv.get("MAILER_PASS", "mail-pw");

    // configuration settings for the SMTP server
    static final Properties props = new Properties() {
        {
            put("mail.smtp.auth", "true");
            put("mail.smtp.starttls.enable", "true");
            put("mail.smtp.EnableSSL.enable", "true");
            put("mail.smtp.host", dotenv.get("MAILER_HOST"));
            put("mail.smtp.port", dotenv.get("MAILER_PORT"));
        }
    };

    // session object from jakarta mail API
    static final Session session = Session.getInstance(props, new jakarta.mail.Authenticator() {
        protected PasswordAuthentication getPasswordAuthentication() {
            return new PasswordAuthentication(username, password);
        }
    });

//    static final Properties props = new Properties() {
//        {
//            put("mail.smtp.host", "sandbox.smtp.mailtrap.io");
//            put("mail.smtp.port", "2525");
//            put("mail.smtp.auth", "true");
//            put("mail.smtp.starttls.enable", "true");
//            put("mail.smtp.starttls.required", "true");
//            put("mail.smtp.ssl.trust", "sandbox.smtp.mailtrap.io");
//            put("mail.username", username);
//            put("mail.password", password);
//        }
//    };

    public static void sendMail(String to, String subject, String textMsg) {
        try {
            // create new email
            Message message = new MimeMessage(session);
            message.setFrom(new InternetAddress(from));
            message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to));
            message.setSubject(subject);
            message.setText(textMsg);
            Transport.send(message);
        } catch (MessagingException e) {
            throw new RuntimeException(e);
        }
    }

    public static void notifyAdminNewSub(String textMsg, String adminJson) {
        System.out.println(adminJson);
        JSONObject resp = new JSONObject(adminJson);
        if (resp.isEmpty()) {
            System.out.println("No admin to notify: " + textMsg);
            return;
        }
//        String adminJson = "[{\"email\":\"ken35kiel@gmail.com\"}]";
//        [{"status":200,"data":["admin@example.com","adm@email.com"]}]
        JSONArray to = resp.getJSONArray("data");
        for (int i = 0; i < to.length(); i++) {
            sendMail(to.getString(i), "New subscription request to Letterpaw SOAP", textMsg);
            System.out.println("Sent email to admin <" + to.getString(i) + "> with message: " + textMsg);
        }

    }
}
