package com.letterpaw.soap.util;

import com.letterpaw.soap.enums.ServiceType;
import io.github.cdimascio.dotenv.Dotenv;
// Make sure to import the correct HttpClient, HttpRequest, and HttpResponse
import java.io.DataOutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.function.Consumer;

public class Request {
    static Dotenv dotenv = Dotenv.configure().ignoreIfMissing().load();

    public enum RequestType {
        GET,
        POST
    }

    public static void GET(ServiceType type, String endpoint, Consumer<String> action) {
        // postData and contentType null because not needed
        request(RequestType.GET, null, null, type, endpoint, action);
    }

    public static void POST(String postData, String contentType, ServiceType type, String endpoint, Consumer<String> action) {
        request(RequestType.POST, postData, contentType, type, endpoint, action);
    }

    public static void request(RequestType reqType, String postData, String contentType, ServiceType type, String endpoint, Consumer<String> respCallback) {
        HttpURLConnection connection = null;

        try {
            // Create connection
            // Get URL
            URL url = new URL("http",
                    dotenv.get((type == ServiceType.PHP ? "PHP_HOST" : "REST_HOST"), "localhost"),
                    Integer.parseInt(dotenv.get((type == ServiceType.PHP ? "PHP_PORT" : "REST_PORT"), "8008")),
                    endpoint);
            System.out.println(url.toString());
            // Open connection
            connection = (HttpURLConnection) url.openConnection();
            // Set request type
            connection.setRequestMethod(reqType.name());
            // Set auth
            connection.setRequestProperty("Authorization", "Bearer " + (
                    type == ServiceType.PHP ? Auth.PHPAPIKey : Auth.RESTAPIKey
            ));

            if (reqType == RequestType.POST) {
                // If POST, postData is written to DataOutputStream
                connection.setRequestProperty("Content-Type",
                        contentType == null ? "application/x-www-form-urlencoded" : contentType);
                connection.setRequestProperty("Content-Length",
                        Integer.toString(postData.getBytes().length));
                connection.setRequestProperty("Content-Language", "en-US");
                connection.setUseCaches(false);
                connection.setDoOutput(true);

                // Send request
                DataOutputStream wr = new DataOutputStream (
                        connection.getOutputStream());
                wr.writeBytes(postData);
                wr.close();
            }

            // Get Response
            InputStreamReader isr = new InputStreamReader(connection.getInputStream());
            BufferedReader rd = new BufferedReader(isr);
            StringBuilder response = new StringBuilder();
            String line;
            while ((line = rd.readLine()) != null) {
                response.append(line);
                response.append('\r');
            }
            rd.close();
            respCallback.accept(response.toString());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.disconnect();
            }
        }
    }
}